mod oyayubi;
pub use oyayubi::OYAYUBI;

use std::fmt::{Display, self};

#[derive(Clone, Copy)]
pub enum Key<'a> {
    Oyayubi {
        latin: char,
        normal: char,
        shift: char,
        alt_shift: Option<char>,
    },
    Single {
        text: &'a str,
        u: f64,
    },
}

pub const fn oyayubi<'a>(latin: char, normal: char, shift: char, alt_shift: Option<char>) -> Key<'a> {
    Key::Oyayubi {
        latin,
        normal,
        shift,
        alt_shift,
    }
}

pub const fn single<'a>(text: &'a str, u: f64) -> Key<'a> {
    Key::Single {
        text,
        u,
    }
}

impl<'a> Key<'a> {
    pub fn width_mod(&self) -> f64 {
        use Key::*;
        match self {
            Oyayubi { .. } => 1.0,
            Single { u, .. } => *u,
        }
    }
}

impl<'a> Display for Key<'a> {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        use Key::*;
        write!(f, "{}", match self {
            Oyayubi { latin, .. } => latin.to_string(),
            Single { text, .. } => text.to_string(),
        })
    }
}